@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 
 import logging
 import os
@@ -149,11 +150,13 @@ class PythonCallbackFunction(ExternalFunction):
         return super(PythonCallbackFunction, self).__call__(self._fcn_id, *args)
 
     def evaluate(self, args):
-        _id = six.next(args) # global callback function identifier
+        if args.__class__ is types.GeneratorType:
+            args = tuple(args)
+        _id = args[0]
         if _id != self._fcn_id:
             raise RuntimeError(
                 "PythonCallbackFunction called with invalid Global ID" )
-        return self._fcn(*tuple(args))
+        return self._fcn(*args[1:])
 
 
 class _ARGLIST(Structure):
