@@ -110,12 +110,13 @@ class LinearDual_PyomoTransformation(Transformation):
         #
         # Add variables
         #
-        # TODO: revisit this hack.  We shouldn't be calling _default()
+        # TODO: revisit this hack.  We shouldn't be calling
+        # _getitem_when_not_present()
         #
         for name, ndx in b_coef:
             v = getattr(dual, name)
             if not ndx in v:
-                v._default(ndx)
+                v._getitem_when_not_present(ndx)
         #
         # Construct the objective
         #
