@@ -31,194 +31,221 @@ from pyomo.dae.misc import get_index_information
 # stored dictionary for up to 10 collocation points.
 try:
     import numpy
-    numpy_available=True
+    numpy_available = True
 except ImportError:
-    numpy_available=False
+    numpy_available = False
     from pyomo.dae.utilities import *
 
 logger = logging.getLogger('pyomo.core')
 
-def _lagrange_radau_transform(v,s):
+
+def _lagrange_radau_transform(v, s):
     ncp = s.get_discretization_info()['ncp']
     adot = s.get_discretization_info()['adot']
+
     def _fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Don't apply this equation at initial point
+        if idx == 0:  # Don't apply this equation at initial point
             raise IndexError("list index out of range")
         low = s.get_lower_element_boundary(i)
         lowidx = tmp.index(low)
-        return sum(v(tmp[lowidx+j])*adot[j][idx-lowidx]*(1.0/(tmp[lowidx+ncp]-tmp[lowidx])) for j in range(ncp+1))
+        return sum(v(tmp[lowidx + j]) * adot[j][idx - lowidx] *
+                   (1.0 / (tmp[lowidx + ncp] - tmp[lowidx]))
+                   for j in range(ncp + 1))
     return _fun
 
-def _lagrange_radau_transform_order2(v,s):
+
+def _lagrange_radau_transform_order2(v, s):
     ncp = s.get_discretization_info()['ncp']
     adotdot = s.get_discretization_info()['adotdot']
+
     def _fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Don't apply this equation at initial point
+        if idx == 0:  # Don't apply this equation at initial point
             raise IndexError("list index out of range")
         low = s.get_lower_element_boundary(i)
         lowidx = tmp.index(low)
-        return sum(v(tmp[lowidx+j])*adotdot[j][idx-lowidx]*(1.0/(tmp[lowidx+ncp]-tmp[lowidx])**2) for j in range(ncp+1))
+        return sum(v(tmp[lowidx + j]) * adotdot[j][idx - lowidx] *
+                   (1.0 / (tmp[lowidx + ncp] - tmp[lowidx]) ** 2)
+                   for j in range(ncp + 1))
     return _fun
 
-def _lagrange_legendre_transform(v,s):
+
+def _lagrange_legendre_transform(v, s):
     ncp = s.get_discretization_info()['ncp']
     adot = s.get_discretization_info()['adot']
+
     def _fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Don't apply this equation at initial point
+        if idx == 0:  # Don't apply this equation at initial point
             raise IndexError("list index out of range")
-        elif i in s.get_finite_elements(): # Don't apply at finite element points
-                                           # continuity equations added later
+        elif i in s.get_finite_elements():  # Don't apply at finite element
+                                            # points continuity equations
+                                            # added later
             raise IndexError("list index out of range")
         low = s.get_lower_element_boundary(i)
         lowidx = tmp.index(low)
-        return sum(v(tmp[lowidx+j])*adot[j][idx-lowidx]*(1.0/(tmp[lowidx+ncp+1]-tmp[lowidx])) for j in range(ncp+1))
+        return sum(v(tmp[lowidx + j]) * adot[j][idx - lowidx] *
+                   (1.0 / (tmp[lowidx + ncp + 1] - tmp[lowidx]))
+                   for j in range(ncp + 1))
     return _fun
 
-def _lagrange_legendre_transform_order2(v,s):
+
+def _lagrange_legendre_transform_order2(v, s):
     ncp = s.get_discretization_info()['ncp']
     adotdot = s.get_discretization_info()['adotdot']
+
     def _fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Don't apply this equation at initial point
+        if idx == 0:  # Don't apply this equation at initial point
             raise IndexError("list index out of range")
-        elif i in s.get_finite_elements(): # Don't apply at finite element points
-                                           # continuity equations added later
+        elif i in s.get_finite_elements():  # Don't apply at finite element
+                                            # points continuity equations
+                                            # added later
             raise IndexError("list index out of range")
         low = s.get_lower_element_boundary(i)
         lowidx = tmp.index(low)
-        return sum(v(tmp[lowidx+j])*adotdot[j][idx-lowidx]*(1.0/(tmp[lowidx+ncp+1]-tmp[lowidx])**2) for j in range(ncp+1))
+        return sum(v(tmp[lowidx + j]) * adotdot[j][idx - lowidx] *
+                   (1.0 / (tmp[lowidx + ncp + 1] - tmp[lowidx]) ** 2) \
+                   for j in range(ncp + 1))
     return _fun
 
-def _hermite_cubic_transform(v,s):
+
+def _hermite_cubic_transform(v, s):
     pass
 
+
 def factorial(n):
-    if n < 0 or not isinstance(n,int):
-        raise ValueError("Can only take the factorial of a non-negative integer")
+    if n < 0 or not isinstance(n, int):
+        raise ValueError(
+            "Can only take the factorial of a non-negative integer")
     elif n > 0:
-        return n*factorial(n-1)
+        return n * factorial(n - 1)
     else:
         return 1
 
-def conv(a,b):
-    if len(a)==0 or len(b)==0:
+
+def conv(a, b):
+    if len(a) == 0 or len(b) == 0:
         raise ValueError("Cannot convolve an empty list")
 
     ans = []
     m = len(a)
     n = len(b)
 
-    for k in range(m+n-1):
+    for k in range(m + n - 1):
         val = 0
-        j = max(0,k-n)
-        stop = min(k,m)
-        while j<=stop:
-             if j<m and (k-j)<n:
-                val += a[j]*b[k-j]
-             j += 1
-        ans.insert(k,val)
+        j = max(0, k - n)
+        stop = min(k, m)
+        while j <= stop:
+            if j < m and (k - j) < n:
+                val += a[j] * b[k - j]
+            j += 1
+        ans.insert(k, val)
 
     return ans
 
-def calc_cp(alpha,beta,k):
+
+def calc_cp(alpha, beta, k):
     gamma = []
-    for i in range(k+1):
-        num = factorial(alpha+k)*factorial(alpha+beta+k+i)
-        denom = factorial(alpha+i)*factorial(k-i)*factorial(i)
-        gamma.insert(i,num/denom)
+    for i in range(k + 1):
+        num = factorial(alpha + k) * factorial(alpha + beta + k + i)
+        denom = factorial(alpha + i) * factorial(k - i) * factorial(i)
+        gamma.insert(i, num / denom)
 
     poly = []
-    for i in range(k+1):
+    for i in range(k + 1):
         if i == 0:
-            poly.insert(i,gamma[i])
+            poly.insert(i, gamma[i])
         else:
             prod = [1]
-            j=1
-            while j<=i:
-                prod=conv(prod,[1,-1])
-                j=j+1
-            while len(poly)<len(prod):
-                poly.insert(0,0)
-            prod = [gamma[i]*t for t in prod]
-            poly = [sum(pair) for pair in zip(poly,prod)]
+            j = 1
+            while j <= i:
+                prod = conv(prod, [1, -1])
+                j += 1
+            while len(poly) < len(prod):
+                poly.insert(0, 0)
+            prod = [gamma[i] * t for t in prod]
+            poly = [sum(pair) for pair in zip(poly, prod)]
 
     cp = numpy.roots(poly)
     return cp
 
+
 def calc_omega(cp):
-    cp.insert
-    a=[]
+
+    a = []
     for i in range(len(cp)):
         ptmp = []
         tmp = 0
         for j in range(len(cp)):
             if j != i:
                 row = []
-                row.insert(0,1/(cp[i]-cp[j]))
-                row.insert(1,-cp[j]/(cp[i]-cp[j]))
-                ptmp.insert(tmp,row)
+                row.insert(0, 1 / (cp[i] - cp[j]))
+                row.insert(1, -cp[j] / (cp[i] - cp[j]))
+                ptmp.insert(tmp, row)
                 tmp += 1
-        p=[1]
-        for j in range(len(cp)-1):
-            p = conv(p,ptmp[j])
+        p = [1]
+        for j in range(len(cp) - 1):
+            p = conv(p, ptmp[j])
         pint = numpy.polyint(p)
         arow = []
         for j in range(len(cp)):
-            arow.append(numpy.polyval(pint,cp[j]))
+            arow.append(numpy.polyval(pint, cp[j]))
         a.append(arow)
     return a
 
-def calc_adot(cp,order=1):
-    a=[]
+
+def calc_adot(cp, order=1):
+    a = []
     for i in range(len(cp)):
         ptmp = []
         tmp = 0
         for j in range(len(cp)):
             if j != i:
                 row = []
-                row.insert(0,1/(cp[i]-cp[j]))
-                row.insert(1,-cp[j]/(cp[i]-cp[j]))
-                ptmp.insert(tmp,row)
+                row.insert(0, 1 / (cp[i] - cp[j]))
+                row.insert(1, -cp[j] / (cp[i] - cp[j]))
+                ptmp.insert(tmp, row)
                 tmp += 1
-        p=[1]
-        for j in range(len(cp)-1):
-            p = conv(p,ptmp[j])
-        pder = numpy.polyder(p,order)
+        p = [1]
+        for j in range(len(cp) - 1):
+            p = conv(p, ptmp[j])
+        pder = numpy.polyder(p, order)
         arow = []
         for j in range(len(cp)):
-            arow.append(numpy.polyval(pder,cp[j]))
+            arow.append(numpy.polyval(pder, cp[j]))
         a.append(arow)
     return a
 
+
 def calc_afinal(cp):
-    afinal=[]
+    afinal = []
     for i in range(len(cp)):
         ptmp = []
         tmp = 0
         for j in range(len(cp)):
             if j != i:
                 row = []
-                row.insert(0,1/(cp[i]-cp[j]))
-                row.insert(1,-cp[j]/(cp[i]-cp[j]))
-                ptmp.insert(tmp,row)
+                row.insert(0, 1 / (cp[i] - cp[j]))
+                row.insert(1, -cp[j] / (cp[i] - cp[j]))
+                ptmp.insert(tmp, row)
                 tmp += 1
-        p=[1]
-        for j in range(len(cp)-1):
-            p = conv(p,ptmp[j])
-        afinal.append(numpy.polyval(p,1.0))
+        p = [1]
+        for j in range(len(cp) - 1):
+            p = conv(p, ptmp[j])
+        afinal.append(numpy.polyval(p, 1.0))
     return afinal
 
+
 class Collocation_Discretization_Transformation(Transformation):
 
-    alias('dae.collocation', doc="Discretizes a DAE model using "\
-          "orthogonal collocation over finite elements transforming "\
+    alias('dae.collocation', doc="Discretizes a DAE model using "
+          "orthogonal collocation over finite elements transforming "
           "the model into an NLP.")
 
     def __init__(self):
@@ -231,17 +258,18 @@ class Collocation_Discretization_Transformation(Transformation):
         self._tau = {}
         self._reduced_cp = {}
         self.all_schemes = {
-            'LAGRANGE-RADAU' : (_lagrange_radau_transform, _lagrange_radau_transform_order2),
-            'LAGRANGE-LEGENDRE' : (_lagrange_legendre_transform,_lagrange_legendre_transform_order2),
-            'HERMITE-CUBIC' : _hermite_cubic_transform,
-            }
+            'LAGRANGE-RADAU': (_lagrange_radau_transform,
+                               _lagrange_radau_transform_order2),
+            'LAGRANGE-LEGENDRE': (_lagrange_legendre_transform,
+                                  _lagrange_legendre_transform_order2),
+            'HERMITE-CUBIC': _hermite_cubic_transform}
 
     def _setup(self, instance):
         instance = instance.clone()
         instance.construct()
         return instance
 
-    def _get_radau_constants(self,currentds):
+    def _get_radau_constants(self, currentds):
         """
         This function sets the radau collocation points and a values depending
         on how many collocation points have been specified and whether or not
@@ -249,8 +277,8 @@ class Collocation_Discretization_Transformation(Transformation):
         """
         if not numpy_available:
             if self._ncp[currentds] > 10:
-                raise ValueError("Numpy was not found so the maximum number of "\
-                    "collocation points is 10")
+                raise ValueError("Numpy was not found so the maximum number "
+                                 "of collocation points is 10")
             self._tau[currentds] = radau_tau_dict[self._ncp[currentds]]
             self._adot[currentds] = radau_adot_dict[self._ncp[currentds]]
             self._adotdot[currentds] = radau_adotdot_dict[self._ncp[currentds]]
@@ -258,40 +286,42 @@ class Collocation_Discretization_Transformation(Transformation):
         else:
             alpha = 1
             beta = 0
-            k = self._ncp[currentds]-1
-            cp = sorted(list(calc_cp(alpha,beta,k)))
-            cp.insert(0,0.0)
+            k = self._ncp[currentds] - 1
+            cp = sorted(list(calc_cp(alpha, beta, k)))
+            cp.insert(0, 0.0)
             cp.append(1.0)
-            adot = calc_adot(cp,1)
-            adotdot = calc_adot(cp,2)
+            adot = calc_adot(cp, 1)
+            adotdot = calc_adot(cp, 2)
 
             self._tau[currentds] = cp
             self._adot[currentds] = adot
             self._adotdot[currentds] = adotdot
             self._afinal[currentds] = None
 
-    def _get_legendre_constants(self,currentds):
+    def _get_legendre_constants(self, currentds):
         """
-        This function sets the legendre collocation points and a values depending
-        on how many collocation points have been specified and whether or not
-        the user has numpy
+        This function sets the legendre collocation points and a values
+        depending on how many collocation points have been specified and
+        whether or not the user has numpy
         """
         if not numpy_available:
             if self._ncp[currentds] > 10:
-                raise ValueError("Numpy was not found so the maximum number of "\
-                    "collocation points is 10")
+                raise ValueError("Numpy was not found so the maximum number "
+                                 "of collocation points is 10")
             self._tau[currentds] = legendre_tau_dict[self._ncp[currentds]]
             self._adot[currentds] = legendre_adot_dict[self._ncp[currentds]]
-            self._adotdot[currentds] = legendre_adotdot_dict[self._ncp[currentds]]
-            self._afinal[currentds] = legendre_afinal_dict[self._ncp[currentds]]
+            self._adotdot[currentds] = \
+                legendre_adotdot_dict[self._ncp[currentds]]
+            self._afinal[currentds] = \
+                legendre_afinal_dict[self._ncp[currentds]]
         else:
             alpha = 0
             beta = 0
             k = self._ncp[currentds]
-            cp = sorted(list(calc_cp(alpha,beta,k)))
-            cp.insert(0,0.0)
-            adot = calc_adot(cp,1)
-            adotdot = calc_adot(cp,2)
+            cp = sorted(list(calc_cp(alpha, beta, k)))
+            cp.insert(0, 0.0)
+            adot = calc_adot(cp, 1)
+            adotdot = calc_adot(cp, 2)
             afinal = calc_afinal(cp)
 
             self._tau[currentds] = cp
@@ -299,7 +329,7 @@ class Collocation_Discretization_Transformation(Transformation):
             self._adotdot[currentds] = adotdot
             self._afinal[currentds] = afinal
 
-    def _get_hermite_constants(self,currentds):
+    def _get_hermite_constants(self, currentds):
         # TODO: finish this
         raise DAE_Error("Not Implemented")
 
@@ -322,50 +352,55 @@ class Collocation_Discretization_Transformation(Transformation):
                       with Radau roots.
         """
 
-        options = kwds.pop('options', {})
-
-        tmpnfe = kwds.pop('nfe',10)
-        tmpncp = kwds.pop('ncp',3)
-        tmpds = kwds.pop('wrt',None)
-        tmpscheme = kwds.pop('scheme','LAGRANGE-RADAU')
+        tmpnfe = kwds.pop('nfe', 10)
+        tmpncp = kwds.pop('ncp', 3)
+        tmpds = kwds.pop('wrt', None)
+        tmpscheme = kwds.pop('scheme', 'LAGRANGE-RADAU')
         self._scheme_name = tmpscheme.upper()
 
         if tmpds is not None:
             if tmpds.type() is not ContinuousSet:
-                raise TypeError("The component specified using the 'wrt' keyword "\
-                     "must be a differential set")
+                raise TypeError("The component specified using the 'wrt' "
+                                "keyword must be a continuous set")
             elif 'scheme' in tmpds.get_discretization_info():
-                raise ValueError("The discretization scheme '%s' has already been applied "\
-                     "to the ContinuousSet '%s'"%(tmpds.get_discretization_info()['scheme'],tmpds.name))
+                raise ValueError("The discretization scheme '%s' has already "
+                                 "been applied to the ContinuousSet '%s'"
+                                 % (tmpds.get_discretization_info()['scheme'],
+                                    tmpds.name))
 
-        if tmpnfe <=0:
-            raise ValueError("The number of finite elements must be at least 1")
+        if tmpnfe <= 0:
+            raise ValueError(
+                "The number of finite elements must be at least 1")
         if tmpncp <= 0:
-            raise ValueError("The number of collocation points must be at least 1")
+            raise ValueError(
+                "The number of collocation points must be at least 1")
 
         if None in self._nfe:
-            raise ValueError("A general discretization scheme has already been applied to "\
-                    "to every differential set in the model. If you would like to specify a "\
-                    "specific discretization scheme for one of the differential sets you must discretize "\
-                    "each differential set individually. If you would like to apply a different "\
-                    "discretization scheme to all differential sets you must declare a new Collocation"\
-                    "_Discretization object")
+            raise ValueError(
+                "A general discretization scheme has already been applied to "
+                "to every differential set in the model. If you would like to "
+                "specify a specific discretization scheme for one of the "
+                "differential sets you must discretize each differential set "
+                "individually. If you would like to apply a different "
+                "discretization scheme to all differential sets you must "
+                "declare a new transformation object")
 
         if len(self._nfe) == 0 and tmpds is None:
             # Same discretization on all differentialsets
             self._nfe[None] = tmpnfe
             self._ncp[None] = tmpncp
             currentds = None
-        else :
-            self._nfe[tmpds.local_name]=tmpnfe
-            self._ncp[tmpds.local_name]=tmpncp
+        else:
+            self._nfe[tmpds.name] = tmpnfe
+            self._ncp[tmpds.name] = tmpncp
             currentds = tmpds.name
 
-        self._scheme = self.all_schemes.get(self._scheme_name,None)
+        self._scheme = self.all_schemes.get(self._scheme_name, None)
         if self._scheme is None:
-            raise ValueError("Unknown collocation scheme '%s' specified using the "\
-                     "'scheme' keyword. Valid schemes are 'LAGRANGE-RADAU', 'LAGRANGE-LEGENDRE'"\
-                     ", and 'HERMITE-CUBIC'" %(tmpscheme))
+            raise ValueError("Unknown collocation scheme '%s' specified using "
+                             "the 'scheme' keyword. Valid schemes are "
+                             "'LAGRANGE-RADAU', 'LAGRANGE-LEGENDRE', and "
+                             "'HERMITE-CUBIC'" % tmpscheme)
 
         if self._scheme_name == 'LAGRANGE-RADAU':
             self._get_radau_constants(currentds)
@@ -373,7 +408,7 @@ class Collocation_Discretization_Transformation(Transformation):
             self._get_legendre_constants(currentds)
 
         for block in instance.block_data_objects(active=True):
-            self._transformBlock(block,currentds)
+            self._transformBlock(block, currentds)
 
         return instance
 
@@ -382,22 +417,24 @@ class Collocation_Discretization_Transformation(Transformation):
         self._fe = {}
         for ds in itervalues(block.component_map(ContinuousSet)):
             if currentds is None or currentds == ds.name:
-                generate_finite_elements(ds,self._nfe[currentds])
+                generate_finite_elements(ds, self._nfe[currentds])
                 if not ds.get_changed():
-                    if len(ds)-1 > self._nfe[currentds]:
-                        print("***WARNING: More finite elements were found in differentialset "\
-                            "'%s' than the number of finite elements specified in apply. "\
-                              "The larger number of finite elements will be used." % (ds.name,))
+                    if len(ds) - 1 > self._nfe[currentds]:
+                        print("***WARNING: More finite elements were found in "
+                              "ContinuousSet '%s' than the number of finite "
+                              "elements specified in apply. The larger number "
+                              "of finite elements will be used." % ds.name)
 
-                self._nfe[ds.name]=len(ds)-1
-                self._fe[ds.name]=sorted(ds)
-                generate_colloc_points(ds,self._tau[currentds])
-                # Adding discretization information to the differentialset object itself
-                # so that it can be accessed outside of the discretization object
+                self._nfe[ds.name] = len(ds) - 1
+                self._fe[ds.name] = sorted(ds)
+                generate_colloc_points(ds, self._tau[currentds])
+                # Adding discretization information to the continuousset
+                # object itself so that it can be accessed outside of the
+                # discretization object
                 disc_info = ds.get_discretization_info()
-                disc_info['nfe']=self._nfe[ds.name]
-                disc_info['ncp']=self._ncp[currentds]
-                disc_info['tau_points']=self._tau[currentds]
+                disc_info['nfe'] = self._nfe[ds.name]
+                disc_info['ncp'] = self._ncp[currentds]
+                disc_info['tau_points'] = self._tau[currentds]
                 disc_info['adot'] = self._adot[currentds]
                 disc_info['adotdot'] = self._adotdot[currentds]
                 disc_info['afinal'] = self._afinal[currentds]
@@ -415,20 +452,23 @@ class Collocation_Discretization_Transformation(Transformation):
                     count = dsets.count(i)
                     if count >= 3:
                         raise DAE_Error(
-                            "Error discretizing '%s' with respect to '%s'. Current implementation "\
-                            "only allows for taking the first or second derivative with respect to "\
-                            "a particular ContinuousSet" %(d.name,i.name))
-                    scheme = self._scheme[count-1]
+                            "Error discretizing '%s' with respect to '%s'. "
+                            "Current implementation only allows for taking the"
+                            " first or second derivative with respect to a "
+                            "particular ContinuousSet" % (d.name, i.name))
+                    scheme = self._scheme[count - 1]
                     # print("%s %s" % (i.name, scheme.__name__))
-                    newexpr = create_partial_expression(scheme,oldexpr,i,loc)
+                    newexpr = create_partial_expression(scheme, oldexpr, i,
+                                                        loc)
                     d.set_derivative_expression(newexpr)
                     if self._scheme_name == 'LAGRANGE-LEGENDRE':
-                        add_continuity_equations(block,d,i,loc)
+                        add_continuity_equations(block, d, i, loc)
 
-            # Reclassify DerivativeVar if all indexing ContinuousSets have been discretized
+            # Reclassify DerivativeVar if all indexing ContinuousSets have
+            # been discretized
             if d.is_fully_discretized():
-                add_discretization_equations(block,d)
-                block.reclassify_component_type(d,Var)
+                add_discretization_equations(block, d)
+                block.reclassify_component_type(d, Var)
 
         # Reclassify Integrals if all ContinuousSets have been discretized
         if block_fully_discretized(block):
@@ -436,13 +476,14 @@ class Collocation_Discretization_Transformation(Transformation):
             if block.contains_component(Integral):
                 for i in itervalues(block.component_map(Integral)):
                     i.reconstruct()
-                    block.reclassify_component_type(i,Expression)
-                # If a model contains integrals they are most likely to appear in the objective
-                # function which will need to be reconstructed after the model is discretized.
+                    block.reclassify_component_type(i, Expression)
+                # If a model contains integrals they are most likely to appear
+                # in the objective function which will need to be reconstructed
+                # after the model is discretized.
                 for k in itervalues(block.component_map(Objective)):
                     k.reconstruct()
 
-    def _get_idx(self,l,t,n,i,k):
+    def _get_idx(self, l, t, n, i, k):
         """
         This function returns the appropriate index for the differential
         and the derivative variables. It's needed because the collocation
@@ -453,14 +494,14 @@ class Collocation_Discretization_Transformation(Transformation):
         """
 
         tmp = t.index(t._fe[i])
-        tik = t[tmp+k]
+        tik = t[tmp + k]
         if n is None:
             return tik
         else:
-            tmpn=n
-            if not isinstance(n,tuple):
+            tmpn = n
+            if not isinstance(n, tuple):
                 tmpn = (n,)
-            return tmpn[0:l]+(tik,)+tmpn[l:]
+            return tmpn[0:l] + (tik,) + tmpn[l:]
 
     def reduce_collocation_points(self, instance, var=None, ncp=None,
                                   contset=None):
@@ -489,47 +530,52 @@ class Collocation_Discretization_Transformation(Transformation):
 
         """
         if contset is None:
-            raise TypeError("A continuous set must be specified using the keyword 'contset'")
+            raise TypeError("A continuous set must be specified using the "
+                            "keyword 'contset'")
         if contset.type() is not ContinuousSet:
-            raise TypeError("The component specified using the 'contset' keyword "\
-                "must be a differential set")
+            raise TypeError("The component specified using the 'contset' "
+                            "keyword must be a ContinuousSet")
         ds = instance.find_component(contset.name)
         if ds is None:
-            raise ValueError("ContinuousSet '%s' is not a valid component of the discretized "\
-                "model instance" %(contset.name))
+            raise ValueError("ContinuousSet '%s' is not a valid component of "
+                             "the discretized model instance" % contset.name)
 
         if len(self._ncp) == 0:
-            raise RuntimeError("This method should only be called after using the apply() method "\
-                "to discretize the model")
+            raise RuntimeError("This method should only be called after using "
+                               "the apply() method to discretize the model")
         elif None in self._ncp:
             tot_ncp = self._ncp[None]
         elif ds.name in self._ncp:
             tot_ncp = self._ncp[ds.name]
         else:
-            raise ValueError("ContinuousSet '%s' has not been discretized yet, please call "\
-                "the apply() method with this ContinuousSet to discretize it before calling "\
-                "this method" %(ds.name))
+            raise ValueError("ContinuousSet '%s' has not been discretized, "
+                             "please call the apply_to() method with this "
+                             "ContinuousSet to discretize it before calling "
+                             "this method" % ds.name)
 
         if var is None:
             raise TypeError("A variable must be specified")
         if var.type() is not Var:
-            raise TypeError("The component specified using the 'var' keyword "\
-                "must be a variable")
+            raise TypeError("The component specified using the 'var' keyword "
+                            "must be a variable")
         tmpvar = instance.find_component(var.name)
         if tmpvar is None:
-            raise ValueError("Variable '%s' is not a valid component of the discretized "\
-                "model instance" %(var.name))
+            raise ValueError("Variable '%s' is not a valid component of the "
+                             "discretized model instance" % var.name)
 
         var = tmpvar
 
         if ncp is None:
-            raise TypeError("The number of collocation points must be specified")
+            raise TypeError(
+                "The number of collocation points must be specified")
         if ncp <= 0:
-            raise ValueError("The number of collocation points must be at least 1")
+            raise ValueError(
+                "The number of collocation points must be at least 1")
         if ncp > tot_ncp:
-            raise ValueError("The number of collocation points used to interpolate "\
-                "an individual variable must be less than the number used to discretize "\
-                "the original model")
+            raise ValueError("The number of collocation points used to "
+                             "interpolate an individual variable must be less "
+                             "than the number used to discretize the original "
+                             "model")
         if ncp == tot_ncp:
             # Nothing to be done
             return instance
@@ -537,58 +583,62 @@ class Collocation_Discretization_Transformation(Transformation):
         # Check to see if the continuousset is an indexing set of the variable
         if var.dim() == 1:
             if ds not in var._index:
-                raise IndexError("ContinuousSet '%s' is not an indexing set of the variable '%s'"\
-                    % (ds.name, var.name))
+                raise IndexError("ContinuousSet '%s' is not an indexing set of"
+                                 " the variable '%s'" % (ds.name, var.name))
         elif ds not in var._index_set:
-            raise IndexError("ContinuousSet '%s' is not an indexing set of the variable '%s'"\
-                             % (ds.name, var.name))
+            raise IndexError("ContinuousSet '%s' is not an indexing set of the"
+                             " variable '%s'" % (ds.name, var.name))
 
         if var.name in self._reduced_cp:
             temp = self._reduced_cp[var.name]
             if ds.name in temp:
-                raise RuntimeError("Variable '%s' has already been constrained to a reduced "\
-                    "number of collocation points over ContinuousSet '%s'.")
+                raise RuntimeError("Variable '%s' has already been constrained"
+                                   " to a reduced number of collocation points"
+                                   " over ContinuousSet '%s'.")
             else:
-                temp[ds.local_name]=ncp
+                temp[ds.name] = ncp
         else:
-            self._reduced_cp[var.local_name] = {ds.local_name: ncp}
+            self._reduced_cp[var.name] = {ds.name: ncp}
 
-        list_name = var.local_name+"_interpolation_constraints"
+        list_name = var.local_name + "_interpolation_constraints"
 
-        instance.add_component(list_name,ConstraintList())
+        instance.add_component(list_name, ConstraintList())
         conlist = instance.find_component(list_name)
 
         t = sorted(ds)
         fe = ds._fe
-        info = get_index_information(var,ds)
+        info = get_index_information(var, ds)
         tmpidx = info['non_ds']
         idx = info['index function']
 
         # Iterate over non_ds indices
         for n in tmpidx:
             # Iterate over finite elements
-            for i in xrange(0,len(fe)-1):
+            for i in xrange(0, len(fe) - 1):
                 # Iterate over collocation points
-                for k in xrange(1,tot_ncp-ncp+1):
+                for k in xrange(1, tot_ncp - ncp + 1):
                     if ncp == 1:
                         # Constant over each finite element
-                        conlist.add(var[idx(n,i,k)]==var[idx(n,i,tot_ncp)])
+                        conlist.add(var[idx(n, i, k)] ==
+                                    var[idx(n, i, tot_ncp)])
                     else:
                         tmp = t.index(fe[i])
-                        tmp2 = t.index(fe[i+1])
-                        ti = t[tmp+k]
-                        tfit = t[tmp2-ncp+1:tmp2+1]
-                        coeff = self._interpolation_coeffs(ti,tfit)
-                        conlist.add(var[idx(n,i,k)]== sum(var[idx(n,i,j)]*coeff.next() for j in xrange(tot_ncp-ncp+1,tot_ncp+1)))
+                        tmp2 = t.index(fe[i + 1])
+                        ti = t[tmp + k]
+                        tfit = t[tmp2 - ncp + 1:tmp2 + 1]
+                        coeff = self._interpolation_coeffs(ti, tfit)
+                        conlist.add(var[idx(n, i, k)] ==
+                                    sum(var[idx(n, i, j)] * coeff.next()
+                                        for j in xrange(tot_ncp - ncp + 1,
+                                                        tot_ncp + 1)))
 
         return instance
 
-    def _interpolation_coeffs(self,ti,tfit):
+    def _interpolation_coeffs(self, ti, tfit):
 
         for i in tfit:
-            l=1
+            l = 1
             for j in tfit:
                 if i != j:
-                    l = l*(ti-j)/(i-j)
+                    l = l * (ti - j) / (i - j)
             yield l
-
