@@ -2,8 +2,8 @@
 #
 #  Pyomo: Python Optimization Modeling Objects
 #  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
-#  Under the terms of Contract DE-NA0003525 with National Technology and 
-#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  Under the terms of Contract DE-NA0003525 with National Technology and
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
@@ -23,91 +23,107 @@ from pyomo.dae.misc import block_fully_discretized
 
 logger = logging.getLogger('pyomo.core')
 
-def _central_transform(v,s):
+
+def _central_transform(v, s):
     """
-    Applies the Central Difference formula of order O(h^2) for first derivatives
+    Applies the Central Difference formula of order O(h^2) for first
+    derivatives
     """
     def _ctr_fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Needed since '-1' is considered a valid index in Python
+        if idx == 0:  # Needed since '-1' is considered a valid index in Python
             raise IndexError("list index out of range")
-        return 1/(tmp[idx+1]-tmp[idx-1])*(v(tmp[idx+1])-v(tmp[idx-1]))
+        return 1 / (tmp[idx + 1] - tmp[idx - 1]) * \
+               (v(tmp[idx + 1]) - v(tmp[idx - 1]))
     return _ctr_fun
 
-def _central_transform_order2(v,s):
+
+def _central_transform_order2(v, s):
     """
-    Applies the Central Difference formula of order O(h^2) for second derivatives
+    Applies the Central Difference formula of order O(h^2) for second
+    derivatives
     """
     def _ctr_fun2(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Needed since '-1' is considered a valid index in Python
+        if idx == 0:  # Needed since '-1' is considered a valid index in Python
             raise IndexError("list index out of range")
-        return 1/((tmp[idx+1]-tmp[idx])*(tmp[idx]-tmp[idx-1]))*(v(tmp[idx+1])-2*v(tmp[idx])+v(tmp[idx-1]))
+        return 1 / ((tmp[idx + 1] - tmp[idx]) * (tmp[idx] - tmp[idx - 1])) * \
+               (v(tmp[idx + 1]) - 2 * v(tmp[idx]) + v(tmp[idx - 1]))
     return _ctr_fun2
 
-def _forward_transform(v,s):
+
+def _forward_transform(v, s):
     """
     Applies the Forward Difference formula of order O(h) for first derivatives
     """
     def _fwd_fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        return 1/(tmp[idx+1]-tmp[idx])*(v(tmp[idx+1])-v(tmp[idx]))
+        return 1 / (tmp[idx + 1] - tmp[idx]) * (v(tmp[idx + 1]) - v(tmp[idx]))
     return _fwd_fun
 
-def _forward_transform_order2(v,s):
+
+def _forward_transform_order2(v, s):
     """
     Applies the Forward Difference formula of order O(h) for second derivatives
     """
     def _fwd_fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        return 1/((tmp[idx+2]-tmp[idx+1])*(tmp[idx+1]-tmp[idx]))*(v(tmp[idx+2])-2*v(tmp[idx+1])+v(tmp[idx]))
+        return 1 / ((tmp[idx + 2] - tmp[idx + 1]) *
+                    (tmp[idx + 1] - tmp[idx])) *\
+               (v(tmp[idx + 2]) - 2 * v(tmp[idx + 1]) + v(tmp[idx]))
     return _fwd_fun
 
-def _backward_transform(v,s):
+
+def _backward_transform(v, s):
     """
     Applies the Backward Difference formula of order O(h) for first derivatives
     """
     def _bwd_fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0: # Needed since '-1' is considered a valid index in Python
+        if idx == 0:  # Needed since '-1' is considered a valid index in Python
             raise IndexError("list index out of range")
-        return 1/(tmp[idx]-tmp[idx-1])*(v(tmp[idx])-v(tmp[idx-1]))
+        return 1 / (tmp[idx] - tmp[idx - 1]) * (v(tmp[idx]) - v(tmp[idx - 1]))
     return _bwd_fun
 
-def _backward_transform_order2(v,s):
+
+def _backward_transform_order2(v, s):
     """
-    Applies the Backward Difference formula of order O(h) for second derivatives
+    Applies the Backward Difference formula of order O(h) for second
+    derivatives
     """
     def _bwd_fun(i):
         tmp = sorted(s)
         idx = tmp.index(i)
-        if idx == 0 or idx == 1: # Needed since '-1' is considered a valid index in Python
+
+        # This check is needed since '-1' is considered a valid index in Python
+        if idx == 0 or idx == 1:
             raise IndexError("list index out of range")
-        return 1/((tmp[idx-1]-tmp[idx-2])*(tmp[idx]-tmp[idx-1]))*(v(tmp[idx])-2*v(tmp[idx-1])+v(tmp[idx-2]))
+        return 1 / ((tmp[idx - 1] - tmp[idx - 2]) *
+                    (tmp[idx] - tmp[idx - 1])) * \
+               (v(tmp[idx]) - 2 * v(tmp[idx - 1]) + v(tmp[idx - 2]))
     return _bwd_fun
 
+
 class Finite_Difference_Transformation(Transformation):
     """
     Transformation that applies finite difference methods to
     DAE, ODE, or PDE models.
     """
-    alias('dae.finite_difference', doc="Discretizes a DAE model using "\
+    alias('dae.finite_difference', doc="Discretizes a DAE model using "
           "a finite difference method transforming the model into an NLP.")
 
-
     def __init__(self):
         super(Finite_Difference_Transformation, self).__init__()
         self._nfe = {}
         self.all_schemes = {
-            'BACKWARD' : (_backward_transform, _backward_transform_order2),
-            'CENTRAL' : (_central_transform, _central_transform_order2),
-            'FORWARD' : (_forward_transform, _forward_transform_order2),
-            }
+            'BACKWARD': (_backward_transform, _backward_transform_order2),
+            'CENTRAL': (_central_transform, _central_transform_order2),
+            'FORWARD': (_forward_transform, _forward_transform_order2)}
 
     def _setup(self, instance):
         instance = instance.clone()
@@ -130,45 +146,45 @@ class Finite_Difference_Transformation(Transformation):
                       scheme is the backward difference method
         """
 
-        options = kwds.pop('options', {})
-
-        tmpnfe = kwds.pop('nfe',10)
-        tmpds = kwds.pop('wrt',None)
-        tmpscheme = kwds.pop('scheme','BACKWARD')
+        tmpnfe = kwds.pop('nfe', 10)
+        tmpds = kwds.pop('wrt', None)
+        tmpscheme = kwds.pop('scheme', 'BACKWARD')
         self._scheme_name = tmpscheme.upper()
 
         if tmpds is not None:
             if tmpds.type() is not ContinuousSet:
-                raise TypeError("The component specified using the 'wrt' keyword "\
-                     "must be a differential set")
+                raise TypeError("The component specified using the 'wrt' "
+                                "keyword must be a continuous set")
             elif 'scheme' in tmpds.get_discretization_info():
-                raise ValueError("The discretization scheme '%s' has already been applied "\
-                     "to the ContinuousSet '%s'" %(tmpds.get_discretization_info()['scheme'],tmpds.name))
+                raise ValueError("The discretization scheme '%s' has already "
+                                 "been applied to the ContinuousSet '%s'" %
+                                 (tmpds.get_discretization_info()['scheme'],
+                                  tmpds.name))
 
         if None in self._nfe:
-            raise ValueError("A general discretization scheme has already been applied to "\
-                    "to every differential set in the model. If you would like to specify a "\
-                    "specific discretization scheme for one of the differential sets you must discretize "\
-                    "each differential set individually. If you would like to apply a different "\
-                    "discretization scheme to all differential sets you must declare a new Implicit_"\
-                    "Euler object")
+            raise ValueError(
+                "A general discretization scheme has already been applied to "
+                "to every continuous set in the model. If you would like to "
+                "apply a different discretization scheme to each continuous "
+                "set, you must declare a new transformation object")
 
         if len(self._nfe) == 0 and tmpds is None:
             # Same discretization on all differentialsets
             self._nfe[None] = tmpnfe
             currentds = None
-        else :
-            self._nfe[tmpds.name]=tmpnfe
+        else:
+            self._nfe[tmpds.name] = tmpnfe
             currentds = tmpds.name
 
-        self._scheme = self.all_schemes.get(self._scheme_name,None)
+        self._scheme = self.all_schemes.get(self._scheme_name, None)
         if self._scheme is None:
-            raise ValueError("Unknown finite difference scheme '%s' specified using the "\
-                     "'scheme' keyword. Valid schemes are 'BACKWARD', 'CENTRAL', and "\
-                     "'FORWARD'" %(tmpscheme))
+            raise ValueError("Unknown finite difference scheme '%s' specified "
+                             "using the 'scheme' keyword. Valid schemes are "
+                             "'BACKWARD', 'CENTRAL', and 'FORWARD'" %
+                             tmpscheme)
 
         for block in instance.block_data_objects(active=True):
-            self._transformBlock(block,currentds)
+            self._transformBlock(block, currentds)
 
         return instance
 
@@ -177,20 +193,22 @@ class Finite_Difference_Transformation(Transformation):
         self._fe = {}
         for ds in itervalues(block.component_map(ContinuousSet)):
             if currentds is None or currentds == ds.name:
-                generate_finite_elements(ds,self._nfe[currentds])
+                generate_finite_elements(ds, self._nfe[currentds])
                 if not ds.get_changed():
-                    if len(ds)-1 > self._nfe[currentds]:
-                        print("***WARNING: More finite elements were found in ContinuousSet "\
-                            "'%s' than the number of finite elements specified in apply. "\
-                            "The larger number of finite elements will be used." %(ds.name))
+                    if len(ds) - 1 > self._nfe[currentds]:
+                        print("***WARNING: More finite elements were found in "
+                              "ContinuousSet '%s' than the number of finite "
+                              "elements specified in apply. The larger number "
+                              "of finite elements will be used." % ds.name)
 
-                self._nfe[ds.name]=len(ds)-1
-                self._fe[ds.name]=sorted(ds)
-                # Adding discretization information to the differentialset object itself
-                # so that it can be accessed outside of the discretization object
+                self._nfe[ds.name] = len(ds) - 1
+                self._fe[ds.name] = sorted(ds)
+                # Adding discretization information to the differentialset
+                # object itself so that it can be accessed outside of the
+                # discretization object
                 disc_info = ds.get_discretization_info()
-                disc_info['nfe']=self._nfe[ds.name]
-                disc_info['scheme']=self._scheme_name + ' Difference'
+                disc_info['nfe'] = self._nfe[ds.name]
+                disc_info['scheme'] = self._scheme_name + ' Difference'
 
         # Maybe check to see if any of the ContinuousSets have been changed,
         # if they haven't then the model components need not be updated
@@ -208,17 +226,20 @@ class Finite_Difference_Transformation(Transformation):
                     count = dsets.count(i)
                     if count >= 3:
                         raise DAE_Error(
-                            "Error discretizing '%s' with respect to '%s'. Current implementation "\
-                            "only allows for taking the first or second derivative with respect to "\
-                            "a particular ContinuousSet" %(d.name,i.name))
-                    scheme = self._scheme[count-1]
-                    newexpr = create_partial_expression(scheme,oldexpr,i,loc)
+                            "Error discretizing '%s' with respect to '%s'. "
+                            "Current implementation only allows for taking the"
+                            " first or second derivative with respect to "
+                            "a particular ContinuousSet" % (d.name, i.name))
+                    scheme = self._scheme[count - 1]
+                    newexpr = create_partial_expression(scheme, oldexpr, i,
+                                                        loc)
                     d.set_derivative_expression(newexpr)
 
-            # Reclassify DerivativeVar if all indexing ContinuousSets have been discretized
+            # Reclassify DerivativeVar if all indexing ContinuousSets have
+            # been discretized
             if d.is_fully_discretized():
-                add_discretization_equations(block,d)
-                block.reclassify_component_type(d,Var)
+                add_discretization_equations(block, d)
+                block.reclassify_component_type(d, Var)
 
         # Reclassify Integrals if all ContinuousSets have been discretized
         if block_fully_discretized(block):
@@ -226,8 +247,9 @@ class Finite_Difference_Transformation(Transformation):
             if block.contains_component(Integral):
                 for i in itervalues(block.component_map(Integral)):
                     i.reconstruct()
-                    block.reclassify_component_type(i,Expression)
-                # If a model contains integrals they are most likely to appear in the objective
-                # function which will need to be reconstructed after the model is discretized.
+                    block.reclassify_component_type(i, Expression)
+                # If a model contains integrals they are most likely to
+                # appear in the objective function which will need to be
+                # reconstructed after the model is discretized.
                 for k in itervalues(block.component_map(Objective)):
                     k.reconstruct()
