@@ -221,11 +221,9 @@ def run(args=None):
         #
         # validate the tree prior to doing anything serious
         #
-        if not scenario_tree.validate():
-            raise RuntimeError("Scenario tree is invalid")
-        else:
-            if options.verbose:
-                print("Scenario tree is valid!")
+        scenario_tree.validate()
+        if options.verbose:
+            print("Scenario tree is valid!")
 
         index_list, num_scenarios_for_solution, num_scenarios_per_sample = \
             partition_scenario_space(scenario_tree,
@@ -396,7 +394,7 @@ def find_candidate(scenario_instance_factory,
 
             with ExtensiveFormAlgorithm(xhat_ph,
                                         options._ef_options,
-                                        prefix="ef_") as ef:
+                                        options_prefix="ef_") as ef:
 
                 ef.build_ef()
                 print("Solving the xhat extensive form.")
@@ -550,7 +548,7 @@ def run_conf(scenario_instance_factory,
             print("")
             gk_ef = ExtensiveFormAlgorithm(gk_ph,
                                            options._ef_options,
-                                           prefix="ef_")
+                                           options_prefix="ef_")
             gk_ef.build_ef()
             print("Solving the xstar extensive form.")
             # Instance preprocessing is managed within the
