@@ -18,7 +18,9 @@ import time
 # these are the only two preprocessors currently invoked by the
 # simple_preprocessor, which in turn is invoked by the preprocess()
 # method of PyomoModel.
-from pyomo.opt import ProblemFormat, PersistentSolver
+from pyomo.core.base.objective import Objective
+from pyomo.core.base.var import Var
+from pyomo.opt import ProblemFormat
 from pyomo.repn.canonical_repn import LinearCanonicalRepn
 from pyomo.repn.compute_canonical_repn import preprocess_block_objectives \
     as canonical_preprocess_block_objectives
@@ -47,7 +49,6 @@ canonical_expression_preprocessor = \
 ampl_expression_preprocessor = \
     pyomo.util.PyomoAPIFactory("pyomo.repn.compute_ampl_repn")
 
-
 #
 # We only want to do the minimal amount of work to get the instance
 # back to a consistent "preprocessed" state. The following attributes
@@ -58,24 +59,21 @@ ampl_expression_preprocessor = \
 #
 class ScenarioTreePreprocessor(PySPConfiguredObject):
 
-    _declared_options = \
-        PySPConfigBlock("Options declared for the "
-                        "ScenarioTreePreprocessor class")
+    @classmethod
+    def _declare_options(cls, options=None):
+        if options is None:
+            options = PySPConfigBlock()
 
-    safe_declare_common_option(_declared_options,
-                               "disable_advanced_preprocessing")
-    safe_declare_common_option(_declared_options,
-                               "preprocess_fixed_variables")
-    safe_declare_common_option(_declared_options,
-                               "symbolic_solver_labels")
+        safe_declare_common_option(options,
+                                   "preprocess_fixed_variables")
+        safe_declare_common_option(options,
+                                   "symbolic_solver_labels")
+        safe_declare_common_option(options,
+                                   "output_times")
+        safe_declare_common_option(options,
+                                   "verbose")
 
-    #
-    # various
-    #
-    safe_declare_common_option(_declared_options,
-                               "output_times")
-    safe_declare_common_option(_declared_options,
-                               "verbose")
+        return options
 
     def __init__(self, *args, **kwds):
 
@@ -105,8 +103,8 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
 
     def add_scenario(self, scenario, scenario_instance, scenario_solver):
 
-        assert scenario._name not in self._scenario_instance
-        assert scenario._name not in self._scenario_to_bundle_map
+        assert scenario.name not in self._scenario_instance
+        assert scenario.name not in self._scenario_to_bundle_map
 
         self._scenario_instance[scenario._name] = scenario_instance
         self._scenario_solver[scenario._name] = scenario_solver
@@ -118,32 +116,38 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         self.all_constraints_updated[scenario._name] = True
         self.constraints_updated_list[scenario._name] = []
 
-        self.objective_updated[scenario._name] = True
-        self.all_constraints_updated[scenario._name] = True
-
-        if not self._options.disable_advanced_preprocessing:
-            scenario_instance = self._scenario_instance[scenario._name]
-            for block in scenario_instance.block_data_objects(active=True):
-                block._gen_obj_ampl_repn = False
-                block._gen_con_ampl_repn = False
-                block._gen_obj_canonical_repn = False
-                block._gen_con_canonical_repn = False
+        scenario_instance = self._scenario_instance[scenario._name]
+        assert scenario_instance is not None
+        for block in scenario_instance.block_data_objects(active=True):
+            assert not hasattr(block, "_gen_obj_ampl_repn")
+            assert not hasattr(block, "_gen_con_ampl_repn")
+            assert not hasattr(block, "_gen_obj_canonical_repn")
+            assert not hasattr(block, "_gen_con_canonical_repn")
+            block._gen_obj_ampl_repn = False
+            block._gen_con_ampl_repn = False
+            block._gen_obj_canonical_repn = False
+            block._gen_con_canonical_repn = False
 
     def remove_scenario(self, scenario):
 
-        assert scenario._name in self._scenario_instance
-        assert scenario._name not in self._scenario_to_bundle_map
+        assert scenario.name in self._scenario_instance
+        assert scenario.name not in self._scenario_to_bundle_map
 
-        if self._options.disable_advanced_preprocessing:
-            scenario_instance = self._scenario_instance[scenario_name]
-            for block in scenario_instance.block_data_objects(active=True):
-                block._gen_obj_ampl_repn = False
-                block._gen_con_ampl_repn = False
-                block._gen_obj_canonical_repn = False
-                block._gen_con_canonical_repn = False
+        scenario_instance = self._scenario_instance[scenario.name]
+        assert scenario_instance is not None
+        for block in scenario_instance.block_data_objects(active=True):
+            assert not block._gen_obj_ampl_repn
+            assert not block._gen_con_ampl_repn
+            assert not block._gen_obj_canonical_repn
+            assert not block._gen_con_canonical_repn
+            del block._gen_obj_ampl_repn
+            del block._gen_con_ampl_repn
+            del block._gen_obj_canonical_repn
+            del block._gen_con_canonical_repn
 
         del self._scenario_instance[scenario._name]
         del self._scenario_solver[scenario._name]
+        del self._scenario_objective[scenario._name]
 
         del self.fixed_variables[scenario._name]
         del self.freed_variables[scenario._name]
@@ -151,9 +155,6 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         del self.all_constraints_updated[scenario._name]
         del self.constraints_updated_list[scenario._name]
 
-        del self.objective_updated[scenario._name]
-        del self.all_constraints_updated[scenario._name]
-
     def add_bundle(self, bundle, bundle_instance, bundle_solver):
 
         assert bundle._name not in self._bundle_instances
@@ -175,7 +176,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         for scenario_name in self._bundle_scenarios[bundle._name]:
             assert scenario_name in self._scenario_instance
             assert scenario_name in self._scenario_to_bundle_map
-            self._scenario_to_bundle_map[scenario_name] = bundle._name
+            del self._scenario_to_bundle_map[scenario_name]
 
         del self._bundle_instances[bundle._name]
         del self._bundle_solvers[bundle._name]
@@ -242,10 +243,10 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         if scenarios is None:
             scenarios = self._scenario_instance.keys()
 
-        if self._options.verbose:
+        if self.get_option("verbose"):
             print("Preprocessing %s scenarios" % len(scenarios))
 
-        if self._options.verbose:
+        if self.get_option("verbose"):
             if len(self._bundle_instances) > 0:
                 print("Preprocessing scenarios without bundles. Bundle "
                       "preprocessing dependencies will be lost. Scenario "
@@ -264,7 +265,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
 
         end_time = time.time()
 
-        if self._options.output_times:
+        if self.get_option("output_times"):
             print("Scenario preprocessing time=%.2f seconds"
                   % (end_time - start_time))
 
@@ -286,7 +287,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         if bundles is None:
             bundles = self._bundle_instances.keys()
 
-        if self._options.verbose:
+        if self.get_option("verbose"):
             print("Preprocessing %s bundles" % len(bundles))
 
         preprocess_bundle_objective = 0b01
@@ -302,7 +303,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
                     preprocess_bundle |= preprocess_bundle_objective
                 if ((len(self.fixed_variables[scenario_name]) > 0) or \
                     (len(self.freed_variables[scenario_name]) > 0)) and \
-                    self._options.preprocess_fixed_variables:
+                    self.get_option("preprocess_fixed_variables"):
                     preprocess_bundle |= \
                         preprocess_bundle_objective | \
                         preprocess_bundle_constraints
@@ -350,11 +351,15 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
 
         end_time = time.time()
 
-        if self._options.output_times:
+        if self.get_option("output_times"):
             print("Bundle preprocessing time=%.2f seconds"
                   % (end_time - start_time))
 
     def _preprocess_scenario(self, scenario_name, solver):
+        # TODO: Does this import need to be delayed because
+        #       it is in a plugins subdirectory
+        from pyomo.solvers.plugins.solvers.persistent_solver import \
+            PersistentSolver
 
         assert scenario_name in self._scenario_instance
         scenario_objective_active = self._scenario_objective[scenario_name].active
@@ -380,20 +385,20 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
            (not instance_all_constraints_updated) and \
            (len(instance_constraints_updated_list) == 0):
             if persistent_solver_in_use:
-                assert solver.instance_compiled()
+                assert solver.has_instance()
 
             # instances are already preproccessed, nothing
             # needs to be done
-            if self._options.verbose:
+            if self.get_option("verbose"):
                 print("No preprocessing necessary for scenario %s"
                       % (scenario_name))
             _cleanup()
             return
 
         if (instance_fixed_variables or instance_freed_variables) and \
-           (self._options.preprocess_fixed_variables):
+           self.get_option("preprocess_fixed_variables"):
 
-            if self._options.verbose:
+            if self.get_option("verbose"):
                 print("Running full preprocessing for scenario %s"
                       % (scenario_name))
 
@@ -409,7 +414,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
 
         if instance_objective_updated:
 
-            if self._options.verbose:
+            if self.get_option("verbose"):
                 print("Preprocessing objective for scenario %s"
                       % (scenario_name))
 
@@ -419,14 +424,18 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
             else:
                 canonical_preprocess_block_objectives(scenario_instance)
 
-            if persistent_solver_in_use and \
-               solver.instance_compiled():
-                solver.compile_objective(scenario_instance)
+            if persistent_solver_in_use and solver.has_instance():
+                obj_count = 0
+                for obj in scenario_instance.component_data_objects(ctype=Objective, descend_into=True, active=True):
+                    obj_count += 1
+                    if obj_count > 1:
+                        raise RuntimeError('Persistent solver interface only supports a single active objective.')
+                    solver.set_objective(obj)
 
         if (instance_fixed_variables or instance_freed_variables) and \
            (persistent_solver_in_use):
 
-            if self._options.verbose:
+            if self.get_option("verbose"):
                 print("Compiling fixed status updates in persistent solver "
                       "for scenario %s" % (scenario_name))
 
@@ -434,16 +443,15 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
             # instance compiled, depending on what state the solver plugin
             # is in relative to the instance.  if this is the case, just
             # don't compile the variable bounds.
-            if solver.instance_compiled():
+            if solver.has_instance():
                 variables_to_change = \
                     instance_fixed_variables + instance_freed_variables
-                solver.compile_variable_bounds(
-                    scenario_instance,
-                    vars_to_update=variables_to_change)
+                for var in variables_to_change:
+                    solver.update_var(var)
 
         if instance_all_constraints_updated:
 
-            if self._options.verbose:
+            if self.get_option("verbose"):
                 print("Preprocessing all constraints for scenario %s"
                       % (scenario_name))
 
@@ -465,7 +473,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
             # TODO
             assert not persistent_solver_in_use
 
-            if self._options.verbose:
+            if self.get_option("verbose"):
                 print("Preprocessing constraint list (size=%s) for "
                       "scenario %s" % (len(instance_constraints_updated_list),
                                        scenario_name))
@@ -491,19 +499,18 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
                     repn_func(constraint_data.body, idMap=idMap)
 
         if persistent_solver_in_use and \
-           (not solver.instance_compiled()):
-             solver.compile_instance(
+           (not solver.has_instance()):
+             solver.set_instance(
                  scenario_instance,
-                 symbolic_solver_labels=self._options.symbolic_solver_labels,
-                 output_fixed_variable_bounds=not self._options.preprocess_fixed_variables)
+                 symbolic_solver_labels=self.get_option("symbolic_solver_labels"),
+                 output_fixed_variable_bounds=not self.get_option("preprocess_fixed_variables"))
 
         _cleanup()
 
     def get_solver_keywords(self):
 
         kwds = {}
-        if not self._options.disable_advanced_preprocessing:
-            if not self._options.preprocess_fixed_variables:
-                kwds['output_fixed_variable_bounds'] = True
+        if not self.get_option("preprocess_fixed_variables"):
+            kwds['output_fixed_variable_bounds'] = True
 
         return kwds
