@@ -799,6 +799,8 @@ def generate_ampl_repn(exp, idMap=None):
 
     if idMap is None:
         idMap = {}
+    if exp is None:
+        return AmplRepn()
     degree = exp.polynomial_degree()
     if (degree is None) or (degree > 1):
         repn = _generate_ampl_repn(exp)
