@@ -347,6 +347,8 @@ class ProblemWriter_bar(AbstractProblemWriter):
             # Fill in the body of the equation
             body_string_buffer = StringIO()
 
+            if constraint_data.body is None:
+                continue
             as_numeric(constraint_data.body).to_string(
                 ostream=body_string_buffer,
                 verbose=False)
@@ -605,7 +607,6 @@ class ProblemWriter_bar(AbstractProblemWriter):
         for block in all_blocks_list:
             tmp = active_components_data_var[id(block)] = \
                   list(obj for obj in block.component_data_objects(Var,
-                                                                   active=True,
                                                                    sort=sorter,
                                                                    descend_into=False))
             create_symbols_func(symbol_map, tmp, labeler)
