@@ -9,7 +9,6 @@
 #  ___________________________________________________________________________
 
 import os
-import copy
 
 import pyutilib.services
 import pyutilib.misc
@@ -110,7 +109,7 @@ class CONOPT(SystemCallSolver):
         #
         # Define command line
         #
-        env=copy.copy(os.environ)
+        env=os.environ.copy()
 
         cmd = [executable, problem_files[0], '-AMPL']
         if self._timer:
