@@ -8,11 +8,11 @@
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
 
+import pyomo.kernel as pmo
 from pyomo.core import ConcreteModel, Param, Var, Expression, Objective, Constraint, NonNegativeReals, maximize
 from pyomo.opt import TerminationCondition
 from pyomo.solvers.tests.models.base import _BaseTestModel, register_model
 
-
 @register_model
 class LP_infeasible2(_BaseTestModel):
     """
@@ -40,9 +40,26 @@ class LP_infeasible2(_BaseTestModel):
     def warmstart_model(self):
         assert self.model is not None
         model = self.model
-        model.x = None
-        model.y = None
+        model.x.value = None
+        model.y.value = None
 
     def post_solve_test_validation(self, tester, results):
-        assert results['Solver'][0]['termination condition'] == TerminationCondition.infeasible
+        if tester is None:
+            assert results['Solver'][0]['termination condition'] == \
+                TerminationCondition.infeasible
+        else:
+            tester.assertEqual(results['Solver'][0]['termination condition'],
+                               TerminationCondition.infeasible)
 
+@register_model
+class LP_infeasible2_kernel(LP_infeasible2):
+
+    def _generate_model(self):
+        self.model = pmo.block()
+        model = self.model
+        model._name = self.description
+
+        model.x = pmo.variable(lb=1)
+        model.y = pmo.variable(lb=1)
+        model.o = pmo.objective(-model.x-model.y, sense=pmo.maximize)
+        model.c = pmo.constraint(model.x+model.y <= 0)
